"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
from pydantic import BaseModel,EmailStr,field_serializer,field_validator
from datetime import datetime
import base64
import binascii
from espy_contact.util.enums import ChannelEnum
class MessageCreate(BaseModel):
    sender_id: int
    recipient_type: ChannelEnum
    recipient_id: int
    content: dict 

    @field_validator('content', mode='before')
    def validate_content(cls, content: dict):
        if not content:
            raise ValueError('Content cannot be empty')
        if not isinstance(content, dict):
            raise ValueError('Content must be a dictionary')
        if 'subject' not in content or content['subject'] == '':
            raise ValueError('Content must have a subject')
        if 'message' not in content or content['message'] == '':
            raise ValueError('Content must have a message')
        return content
    class Config:
        json_schema_extra = {
            "example": {
                "sender_id": 1,
                "recipient_type": ChannelEnum.SCHOOL,
                "recipient_id": 2,
                "event_type": "School Holiday",
                "content": {
                    "subject": "Test",
                    "message": "This is a test message"
                }
            }
        }


class MessageResponse(MessageCreate):
    updated_at: datetime
    id: int

    @field_serializer('updated_at')
    def serialize_updated_at(self, updated_at: datetime):
        return updated_at.isoformat()

class AttachmentDto(BaseModel):
    filename: str
    content: str # base64 encoded
    mime_type: str | None = None

    @field_validator('content')
    def validate_content(cls, content: str):
        if not content:
            raise ValueError('Content cannot be empty')
        try:
            base64.b64decode(content, validate=True)
        except binascii.Error as e:
            raise ValueError('Content must be base64 encoded')
        return content
    
class MailerDto(BaseModel):
    recipients: list[EmailStr]
    name: str | None = None
    subject: str
    message: str
    created_at: datetime | None = datetime.now()
    is_html: bool | None = False
    attachments: list[AttachmentDto] | None = None
class Notification(MessageCreate):
    event_type: str
    created_at: datetime | None = datetime.now()


"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
from pydantic import BaseModel,Field
from typing import List,Optional,Any
from typing_extensions import Annotated
from pydantic.functional_validators import BeforeValidator
from datetime import datetime
from .schema import UserResponse
from bson import ObjectId


PyObjectId = Annotated[str, BeforeValidator(str)]
# class UserModel(BaseModel):
#     """ Horace User record."""
#     id: Optional[PyObjectId] = Field(alias="_id", default=None)
#     firstname: Optional[str] = None
#     lastname: Optional[str] = None
#     email: Optional[str] = None
#     password: Optional[str] = None
#     active: bool = False
class UserCol(BaseModel):
    users: List[UserResponse]

class Course(BaseModel):
    id: Optional[ObjectId] = Field(default_factory=ObjectId, alias='_id')
    author: UserResponse
    course_name: str = Field(..., alias='courseName')
    category: str
    target: str
    curriculum: Any
    brief: str
    price: float
    tax: float
    created_on: datetime = Field(default_factory=datetime.now, alias='createdOn')
    thumbnail: Optional[str] = None
    updated_on: Optional[datetime] = Field(None, alias='updatedOn')
    total_steps: int = Field(..., alias='totalSteps')
    draft: bool

    class Config:
        populate_by_name = True
        arbitrary_types_allowed=True
        json_schema_extra = {
            "example": {
                "author": {
                    # Assuming some example fields for Appuser
                },
                "course_name": "Introduction to Python",
                "category": "Programming",
                "target": "Beginner programmers",
                "curriculum": {
                    "introduction": "Welcome to the course",
                    "contents": [
                        {"module": "Basics", "lessons": ["Setup", "Syntax"]}
                    ]
                },
                "brief": "Learn Python from scratch!",
                "price": 19.99,
                "tax": 1.99,
                "thumbnail": "url_to_thumbnail_image",
                "total_steps": 10,
                "draft": False
            }
        }

class AssetCount(BaseModel):
    quiz_count: int
    lab_count: int
    lesson_count: int
    download_count: int
    note_count: int

class Lecture(BaseModel):
    id: int
    title: str
    video: str
    type: str
    slide: str

class Quiz(BaseModel):
    # Define the properties of Quiz here
    questions: List[str]

class Note(BaseModel):
    # Define the properties of Note here
    content: str

class Section(BaseModel):
    title: str
    description: str
    id: str
    lecture: List[Lecture]
    quiz: Optional[Quiz] = None
    note: Optional[Note] = None

class CurriculumMap(BaseModel):
    section: List[Section]
    requirement: List[str]
    objective: List[str]

class Post(BaseModel):
    id: str
    user: str
    message: str
    type: str
    course: str
    created_on: datetime = Field(default_factory=datetime.now, alias='createdOn')
    modified_on: datetime = Field(default_factory=datetime.now, alias='modifiedOn')
    like: int
    share: int
    rating: int

    class Config:
        populate_by_name = True
        json_encoders = {
            datetime: lambda dt: dt.isoformat()
        }
        json_schema_extra = {
            "example": {
                "id": "12345",
                "user": "user123",
                "message": "This is a post message.",
                "type": "info",
                "course": "course123",
                "createdOn": "2023-01-01T12:00:00Z",
                "modifiedOn": "2023-01-02T12:00:00Z",
                "like": 10,
                "share": 5,
                "rating": 4
            }
        }
class PostResponse(BaseModel):
    id: str
    user: str
    message: str
    type: str
    course: str
    created_on: datetime = Field(default_factory=datetime.now, alias='createdOn')
    modified_on: datetime = Field(default_factory=datetime.now, alias='modifiedOn')
    like: int
    share: int
    rating: int

    class Config:
        # Use alias generation to maintain the Python convention in code and the original Java naming in JSON
        populate_by_name = True
        from_attributes = True  # Enable ORM mode if integrating with ORMs
        json_schema_extra = {
            "example": {
                "id": "post123",
                "user": "user456",
                "message": "This is a sample post message.",
                "type": "announcement",
                "course": "course789",
                "createdOn": "2023-01-01T12:00:00Z",
                "modifiedOn": "2023-01-02T12:00:00Z",
                "like": 100,
                "share": 50,
                "rating": 5
            }
        }

class CourseResponse(BaseModel):
    course_id: str
    course_name: str
    author: UserResponse
    category: str
    target: str
    curriculum: CurriculumMap
    brief: str
    price: float
    tax: float
    created_on: str
    thumbnail: str
    total_steps: int
    draft: bool
    posts: List[PostResponse]
    asset_count: AssetCount

class CourseItem(BaseModel):
    id: Optional[PyObjectId] = Field(alias="id", default=None)
    course_name: Optional[str] = None
    brief: str
    updated_on: Optional[datetime] = None
    thumbnail: Optional[str] = None
    category: Optional[str]
    total_steps: Optional[int] = 0
    draft: Optional[bool]
    author: Optional[UserResponse] = None
    students: Optional[int] = 0
    posts: Optional[List[Post]] = None
    cost: Optional[float] = 0

class Post(BaseModel):
    # Define Post model attributes as needed
    pass

class Author(BaseModel):
    first_name: str
    last_name: str

    def full_name(self):
        return f"{self.first_name} {self.last_name}"





"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
from datetime import datetime, date
from typing import Optional
from pydantic import BaseModel, Field, model_validator,confloat
from espy_contact.util.enums import StatusEnum, NigerianBank
class AccountDto(BaseModel):
    id: Optional[int] = None
    bank: NigerianBank
    account_name: str
    account_number: str
    currency: str
    is_active: bool
    account_officer: str
    account_admin: str
    created: datetime
    modified: datetime

class FeeCreate(BaseModel):
    classroom_id: int
    fee_name: str
    amount: confloat(ge=0) = Field(..., description="Amount is zero if auto created")
    due_date: date = Field(description="Due date for fee payment")
    start_date: date = Field(description="Start date for fee payment", default=date.today())
    status: StatusEnum | None = None
    creator_id: int
    @model_validator(mode='after')
    def validate_dates(self: 'FeeDto') -> 'FeeDto':
        due_date = self.due_date
        start_date = self.start_date
        today = date.today()
        if due_date is not None:
            if due_date <= today:
                raise ValueError("Due date must be greater than today")
            if start_date is not None and due_date <= start_date:
                raise ValueError("Due date must be greater than start date")
        
        if start_date is not None and start_date < today:
            raise ValueError("Start date cannot be in the past")

        return self


class FeeDto(FeeCreate):
    id: int
    created_on: datetime | None = None
    modified_on: datetime | None = None



from threading import local

class Config(local):
    def __init__(self):
        self.schema = "campus"

config = Config()

def get_current_schema():
    return config.schema

def set_current_schema(schema):
    config.schema = schema